# This source file is part of the Aument language
# Copyright (c) 2021 the aument contributors
#
# Licensed under Apache License v2.0 with Runtime Library Exception
# See LICENSE.txt for license information
left_pad = 12
line_pad = '    '

commands = [
    (
        "build",
        "builds source code into a single binary",
        "input-file output-file",
        """\
Builds *input-file* into an executable binary named *output-file*.

aument compiles code by invoking a C compiler. By default, this is *gcc*,
however you can have aument use another compiler by specifying it in
the `CC` environment variable.

Unless the `--no-opt` parameter is passed, aument invokes the C compiler
with the following arguments:

```
-flto -O2
```

Passing `-b` will make aument output bytecode before it is compiled to C.

Passing `-c` will make aument write C code into *output-file* instead
of outputting a compiled binary.

Passing `-g` will add the `-g` flag to the C compiler call.\
""",
    ),
    (
        "run",
        "runs a program",
        "input-file",
        """\
Runs *input-file* through an interpreter.

Passing `-b` will make aument output bytecode before it is interpreted.\
""",
    ),
    (
        "help",
        "shows this help screen",
        "[command]",
        """\
Shows the documentation for a specified command if given.
Else, a general help screen is shown.\
""",
    ),
    (
        "version",
        "print aument version",
        "",
        "Prints aument's current version number."
    )
]
commands.sort(key=lambda x: x[0])

text = {}

main_template = """\
Usage:
    aument [command] [options] file...

Commands:
%s

Use aument help [command] for more information.

Copyright (c) 2021 the aument contributors.
Project link: https://github.com/aument-lang/aument
"""

text["main"] = main_template % ('\n'.join(map(lambda x: line_pad + x[0].ljust(left_pad) + x[1],commands)))

for cmd in commands:
    name, summary, args, doc = cmd
    text[name] = f"""\
Usage:
    aument {name} {args} {args}

Summary:
    {doc}
"""

c_src = "/* This file is generated by gen_help.py. Please don't modify it. */\n#pragma once\n#include <string.h>\n\n"
for k,v in text.items():
    c_src += ("static const char *AU_HELP_%s = \"%s\";\n"%(k.upper(), v.encode('unicode_escape').decode('ascii').replace('"', '\\"')))

c_src += "static const char *help_text(const char *action) {"
for k in text:
    c_src += "if(strcmp(action,\"%s\")==0)return AU_HELP_%s;"%(k,k.upper())
c_src += "return AU_HELP_MAIN;}"

with open("src/help.h", "w") as f:
    f.write(c_src)

md_src = """\
# Command line options

aument can be invoked in the following way:

```
aument [command] [options] file...
```

This section documents aument's commands.

The section below was generated automatically (devs: *gen_help.py*).
Please don't modify it by hand!

"""
for cmd in commands:
    name, summary, args, doc = cmd
    md_src += f"""\
## `{name}`: {summary}

### Usage

```
aument {name} {args}
```

### Description

{doc}

"""

with open("docs/cmdline.md", "w") as f:
    f.write(md_src)
